@@ -183,12 +183,19 @@ class Command(object):
         
         return parser
 
+    def handle(self, app, *args, **kwargs):
+        """
+        Handles the command with given app. Default behaviour is to call within
+        a test request context.
+        """
+        with app.test_request_context():
+            self.run(*args, **kwargs)
+
     def run(self):
 
         """
-        Runs a command. This must be implemented by the subclass. The first
-        argument is always the app (Flask instance) followed by arguments
-        as configured by the Command options.
+        Runs a command. This must be implemented by the subclass. Should take
+        arguments as configured by the Command options.
         """
 
         raise NotImplementedError
@@ -224,10 +231,8 @@ class Shell(Command):
 
     :param banner: banner appearing at top of shell when started
     :param make_context: a callable returning a dict of variables 
-                         used in the shell namespace. The callable 
-                         takes a single argument, "app", the Flask 
-                         instance. By default returns a dict consisting
-                         of just the app.
+                         used in the shell namespace. By default 
+                         returns a dict consisting of just the app.
     :param use_ipython: use IPython shell if available, ignore if not. 
                         The IPython shell can be turned off in command 
                         line by passing the **--no-ipython** flag.
@@ -298,22 +303,25 @@ class Server(Command):
                          This can be overriden in the command line by 
                          passing the **-r** flag.
                          
+    :param options: :func:`werkzeug.run_simple` options.
     """
 
     description = 'Runs the Flask development server i.e. app.run()'
 
     def __init__(self, host='127.0.0.1', port=5000, use_debugger=True,
-        use_reloader=True):
+        use_reloader=True, **options):
 
 
         self.port = port
         self.host = host
         self.use_debugger = use_debugger
         self.use_reloader = use_reloader
+        self.server_options = options
     
     def get_options(self):
 
-        return (
+        options = (
+
                 Option('-t', '--host',
                        dest='host',
                        default=self.host),
@@ -323,23 +331,44 @@ class Server(Command):
                        type=int,
                        default=self.port),
 
-                Option('-d', '--debug',
-                       action='store_true',
-                       dest='use_debugger',
-                       default=self.use_debugger),
-        
-                Option('-r', '--reload',
-                       action='store_true',
-                       dest='use_reloader',
-                       default=self.use_reloader))
+        ) 
+
+        if self.use_debugger:
+            options += (Option('-d', '--no-debug',
+                               action='store_false',
+                               dest='use_debugger',
+                               default=self.use_debugger),)
+
+        else:
+            options += (Option('-d', '--debug',
+                               action='store_true',
+                               dest='use_debugger',
+                               default=self.use_debugger),)
+
+        if self.use_reloader:
+            options += (Option('-r', '--no-reload',
+                               action='store_false',
+                               dest='use_reloader',
+                               default=self.use_reloader),)
+
+        else:
+            options += (Option('-r', '--reload',
+                               action='store_true',
+                               dest='use_reloader',
+                               default=self.use_reloader),)
+
+        return options
+
+    def handle(self, app, host, port, use_debugger, use_reloader):
+        # we don't need to run the server in request context
+        # so just run it directly
 
-    def run(self, host, port, use_debugger, use_reloader):
-        app = _request_ctx_stack.top.app
         app.run(host=host,
                 port=port,
                 debug=use_debugger,
                 use_debugger=use_debugger,
-                use_reloader=use_reloader)
+                use_reloader=use_reloader,
+                **self.server_options)
 
 
 class InvalidCommand(Exception):
@@ -355,7 +384,7 @@ class Manager(object):
         
         class Print(Command):
 
-            def run(self, app):
+            def run(self):
                 print "hello"
 
         app = Flask(__name__)
@@ -438,7 +467,9 @@ class Manager(object):
 
             manager = Manager(create_app)
             manager.add_option("-c", "--config", dest="config", required=False)
-            
+           
+        and are evoked like this::
+
             > python manage.py -c dev.cfg mycommand
 
         Any manager options passed in the command line will not be passed to 
@@ -453,8 +484,7 @@ class Manager(object):
         """
         Adds a command function to the registry.
         
-        :param func: command function. Should take at least one argument, the 
-                     Flask application. Additional arguments depend on the 
+        :param func: command function.Arguments depend on the 
                      options.
         
         """
@@ -484,11 +514,12 @@ class Manager(object):
                     options.append(Option('-%s' % arg[0],
                                           '--%s' % arg,
                                           dest=arg,
+                                          type=unicode,
                                           required=False,
                                           default=default))
         
             else:
-                options.append(Option(arg))
+                options.append(Option(arg, type=unicode))
 
 
         command = Command()
@@ -528,7 +559,7 @@ class Manager(object):
 
             @option('-n', '--name', dest='name')
             @option('-u', '--url', dest='url')
-            def hello(app, name, url):
+            def hello(name, url):
                 print "hello", name, url
 
         Takes the same arguments as the ``Option`` constructor.
@@ -568,14 +599,15 @@ class Manager(object):
         Returns string consisting of all commands and their
         descriptions.
         """
+        pad = max(map(len, self._commands.iterkeys())) + 2
+        format = '  %%- %ds%%s' % pad
 
         rv = []
 
         for name, command in self._commands.iteritems():
             usage = name
-            description = command.description
-            if description:
-                usage += ": " + description
+            description = command.description or ''
+            usage = format % (name, description)
             rv.append(usage)
 
         return "\n".join(rv)
@@ -610,14 +642,19 @@ class Manager(object):
                 remaining_args.append(arg)
 
         command_parser = command.create_parser(prog + " " + name)
-        command_namespace = command_parser.parse_args(remaining_args)
+        if getattr(command, 'capture_all_args', False):
+            command_namespace, unparsed_args = \
+                command_parser.parse_known_args(remaining_args)
+            positional_args = [unparsed_args]
+        else:
+            command_namespace = command_parser.parse_args(remaining_args)
+            positional_args = []
         
         app = self.create_app(**app_namespace.__dict__)
 
-        with app.test_request_context():
-            command.run(**command_namespace.__dict__)
+        command.handle(app, *positional_args, **command_namespace.__dict__)
 
-    def run(self, commands=None):
+    def run(self, commands=None, default_command=None):
         
         """
         Prepares manager to receive command line input. Usually run
@@ -625,14 +662,25 @@ class Manager(object):
 
         :param commands: optional dict of commands. Appended to any commands 
                          added using add_command().
+
+        :param default_command: name of default command to run if no 
+                                arguments passed.
         """
 
         if commands:
             self._commands.update(commands)
-        
+
         try:
+            if len(sys.argv) == 1 and default_command is not None:
+                command = default_command
+            else:
+                command = sys.argv[1]
+
+            if command is None:
+                raise InvalidCommand, "Please provide a command"
+
             self.handle(sys.argv[0],
-                        sys.argv[1],
+                        command,
                         sys.argv[2:])
             
             sys.exit(0)
