@@ -56,6 +56,18 @@ class CommandWithDynamicOptions(Command):
         print name
 
 
+class CommandWithCatchAll(Command):
+    "command with catch all args"
+
+    capture_all_args = True
+
+    def get_options(self):
+        return (Option('--foo', dest='foo',
+                       action='store_true'),)
+    def run(self, remaining_args, foo):
+        print remaining_args
+
+
 class TestCommands(unittest.TestCase):
 
     TESTING = True
@@ -184,8 +196,10 @@ class TestManager(unittest.TestCase):
         @manager.command
         def hello(name, url=None):
             if url:
+                assert type(url) is unicode
                 print "hello", name, "from", url
             else:
+                assert type(name) is unicode
                 print "hello", name
         
         assert 'hello' in manager._commands
@@ -237,7 +251,7 @@ class TestManager(unittest.TestCase):
         manager = Manager(self.app)
         manager.add_command("simple", SimpleCommand())
 
-        assert "simple: simple command" in manager.get_usage()
+        assert "simple     simple command" in manager.get_usage()
 
     def test_run_existing_command(self):
         
@@ -315,6 +329,16 @@ class TestManager(unittest.TestCase):
             assert e.code == 0
         assert "Fred" in sys.stdout.getvalue()
 
+    def test_run_catch_all(self):
+        manager = Manager(self.app)
+        manager.add_command("catch", CommandWithCatchAll())
+        sys.argv = ["manage.py", "catch", "pos1", "--foo", "pos2", "--bar"]
+        try:
+            manager.run()
+        except SystemExit, e:
+            assert e.code == 0
+        assert "['pos1', 'pos2', '--bar']" in sys.stdout.getvalue()
+
     def test_run_bad_options(self):
 
         manager = Manager(self.app)
@@ -335,3 +359,12 @@ class TestManager(unittest.TestCase):
         manager = Manager(lambda: app)
         assert callable(manager.app)
 
+    def test_run_with_default_command(self):
+        manager = Manager(self.app)
+        manager.add_command('simple', SimpleCommand())
+        try:
+            manager.run(default_command='simple')
+        except SystemExit, e:
+            assert e.code==0
+        assert 'OK' in sys.stdout.getvalue()
+
